@@ -12,7 +12,7 @@ def recursion_lock(retval="<recursion detected>", lock_name="__recursion_lock__"
             try:
                 return func(self, *args, **kw)
             finally:
-                setattr(self, lock_name, False)
+                delattr(self, lock_name)
 
         wrapper.__name__ = func.__name__
         return wrapper
@@ -22,17 +22,38 @@ def recursion_lock(retval="<recursion detected>", lock_name="__recursion_lock__"
 
 class Container(dict):
     r"""
-    A generic container of attributes.
+    Generic ordered dictionary that allows both key and attribute access.
 
-    Containers are dictionaries, translating attribute access into key access, and preserving key order.
+    Containers are dictionaries, translating attribute access into key access, and preserving key order. Also they use call method to add keys, because **kw does not preserve order.
+
+    Structs parse return containers, becuase their fields have order.
+
+    Example::
+
+        Container([("name","anonymous"),("age",21)])
+        
+        Container(name="anonymous")(age=21)
+
+        # This is NOT correct because keyword arguments order is not preserved.
+        Container(name="anonymous",age=21)
+
+        Container(container2tocopy)
     """
     __slots__ = ["__keys_order__","__recursion_lock__"]
 
     def __init__(self, *args, **kw):
         object.__setattr__(self, "__keys_order__", [])
-        for arg in args:
-            for k, v in arg.items():
+        if isinstance(args, dict):
+            for k, v in args.items():
                 self[k] = v
+            return
+        for arg in args:
+            if isinstance(arg, dict):
+                for k, v in arg.items():
+                    self[k] = v
+            else:
+                for k, v in arg:
+                    self[k] = v
         for k, v in kw.items():
             self[k] = v
 
@@ -54,33 +75,86 @@ class Container(dict):
     __delattr__ = __delitem__
     __setattr__ = __setitem__
 
+    def __call__(self, **kw):
+        """
+        Chains adding new entries to the same container. See ctor.
+        """
+        for k,v in kw.items():
+            self.__setitem__(k, v)
+        return self
+
     def clear(self):
         dict.clear(self)
         del self.__keys_order__[:]
 
     def pop(self, key, *default):
+        """
+        Removes and returns the value for a given key, raises KeyError if not found.
+        """
         val = dict.pop(self, key, *default)
         self.__keys_order__.remove(key)
         return val
 
     def popitem(self):
-        k, v = dict.popitem(self)
-        self.__keys_order__.remove(k)
+        """
+        Removes and returns the last key and value from order.
+        """
+        k = self.__keys_order__.pop()
+        v = dict.pop(k)
         return k, v
 
-    def update(self, seq, **kw):
-        if hasattr(seq, "keys"):
-            for k in seq.keys():
-                self[k] = seq[k]
+    def update(self, seqordict, **kw):
+        if isinstance(seqordict, dict):
+            for k, v in seqordict.items():
+                self[k] = v
         else:
-            for k, v in seq:
+            for k, v in seqordict:
                 self[k] = v
         dict.update(self, kw)
 
     def copy(self):
-        inst = self.__class__()
-        inst.update(self.iteritems())
-        return inst
+        return Container(self.iteritems())
+
+    __update__ = update
+    __copy__ = copy
+
+    def iterkeys(self):
+        return iter(self.__keys_order__)
+
+    def itervalues(self):
+        return (self[k] for k in self.__keys_order__)
+
+    def iteritems(self):
+        return ((k, self[k]) for k in self.__keys_order__)
+
+    def keys(self):
+        return self.__keys_order__
+
+    def values(self):
+        return list(self.itervalues())
+
+    def items(self):
+        return list(self.iteritems())
+
+    __iter__ = iterkeys
+
+    def __eq__(self, other, skiporder=False):
+        if not isinstance(other, dict):
+            return False
+        if len(self) != len(other):
+            return False
+        if skiporder:
+            for k,v in self.iteritems():
+                if k not in other or v != other[k]:
+                    return False
+        else:
+            for (k,v),(k2,v2) in zip(self.iteritems(), other.iteritems()):
+                if k != k2 or v != v2:
+                    return False
+        return True
+
+    def __ne__(self, other, skiporder=False):
+        return not self.__eq__(other, skiporder)
 
     def _search(self, name, search_all):
         items = []
@@ -111,31 +185,15 @@ class Container(dict):
     def search_all(self, name):
         return self._search(name, True)
 
-    __update__ = update
-    __copy__ = copy
-
-    def __iter__(self):
-        return iter(self.__keys_order__)
-
-    iterkeys = __iter__
-
-    def itervalues(self):
-        return (self[k] for k in self.__keys_order__)
-
-    def iteritems(self):
-        return ((k, self[k]) for k in self.__keys_order__)
-
-    def keys(self):
-        return self.__keys_order__
-
-    def values(self):
-        return list(self.itervalues())
-
-    def items(self):
-        return list(self.iteritems())
-
+    @recursion_lock()
     def __repr__(self):
-        return "%s(%s)" % (self.__class__.__name__, dict.__repr__(self))
+        parts = ["Container"]
+        for k,v in self.iteritems():
+            if not k.startswith("_"):
+                parts.extend(["(",str(k),"=",repr(v),")"])
+        if len(parts) == 1:
+            parts.append("()")
+        return "".join(parts)
 
     @recursion_lock()
     def __str__(self, indentation="\n    "):
@@ -143,8 +201,7 @@ class Container(dict):
         for k,v in self.iteritems():
             if not k.startswith("_"):
                 text.extend([indentation, k, " = "])
-                lines = str(v).split("\n")
-                text.append(indentation.join(lines))
+                text.append(indentation.join(str(v).split("\n")))
         return "".join(text)
 
 
@@ -155,6 +212,12 @@ class FlagsContainer(Container):
     Only set flags are displayed.
     """
 
+    def __eq__(self, other, skiporder=True):
+        return super(FlagsContainer, self).__eq__(other, skiporder)
+
+    def __ne__(self, other, skiporder=True):
+        return not self.__eq__(other, skiporder)
+
     @recursion_lock()
     def __str__(self, indentation="\n    "):
         text = ["FlagsContainer: "]
@@ -224,13 +287,8 @@ class LazyContainer(object):
         return not (self == other)
 
     def __str__(self):
-        return self.__pretty_str__()
-
-    def __pretty_str__(self, nesting=1, indentation="    "):
         if self._value is NotImplemented:
             text = "<unread>"
-        elif hasattr(self._value, "__pretty_str__"):
-            text = self._value.__pretty_str__(nesting, indentation)
         else:
             text = str(self._value)
         return "%s: %s" % (self.__class__.__name__, text)
